extensions [palette]

globals [
  building1-coordinates
  building2-coordinates

  shelter-coordinates-top    ; New shelter coordinates
  shelter-coordinates-bottom ; New shelter coordinates

  in-shelter-top-count      ; Track people in top shelter
  in-shelter-bottom-count   ; Track people in bottom shelter
  shelter-top-occupants   ; Current count in top shelter
  shelter-bottom-occupants ; Current count in bottom shelter

  missile-targets
  explosion-areas
  impact-areas
  casualties
  injured-count
  floor-colors
  exits
  missiles-launched
  pending-explosions
  simulation-complete
  initial-reaction-percentage  ; Store initial reaction percentage
  first-hit                   ; Track if first missile has hit

  early-evacuees        ; People who left before first missile hit
  late-evacuees         ; People who left after first missile hit
  shelter-reached       ; People who made it to shelter
  building-stayers      ; People who chose to stay in building
]

breed [building-occupants building-occupant]
breed [missiles missile]
breed [explosions explosion]
breed [exit-visuals exit-visual]
breed [floor-labels floor-label]

exit-visuals-own [
  floor-number
  building-number
]

building-occupants-own [
  in-building?      ; Whether currently in building
  in-shelter?       ; Whether in shelter
  health            ; Health status

  target            ; Current target (exit or person to follow)
  following-who     ; Who am I following (nobody if going to exit)

  building-number   ; Which building
  floor-number      ; Which floor
  evacuation-speed  ; Movement speed

  reacting?         ; Whether reacting to alarm
  next-exit         ; Next chosen exit
  waiting-time      ; Time spent waiting
  switch-chance     ; Probability of switching exits (starts at 0.5)

  chosen-shelter    ; "top" or "bottom"
  shelter-attempt-count  ; Track how many times tried to enter a full shelter
  waiting-at-shelter?    ; Whether currently waiting outside shelter

  evacuation-decision   ; "shelter", "stay", or "none"
  evacuation-timing     ; "early", "late", or "none"
]

missiles-own [
  impact-point
  explosion-radius
  speed
]

explosions-own [
  duration
  radius
]

to setup
  clear-all
  set-colors
  setup-globals
  setup-buildings
  setup-shelter
  setup-people
  set first-hit false
  set initial-reaction-percentage reaction-percentage
  set missiles-launched 0
  set simulation-complete false
  reset-ticks
end

to set-colors
  ; New dynamic color generation based on number-of-floors
  set floor-colors []
  let base-color 140  ; Darkest blue
  let color-step (255 - base-color) / (number-of-floors - 1)

  let i 0
  repeat number-of-floors [
    let current-color base-color + (i * color-step)
    set floor-colors lput (list current-color current-color 255) floor-colors
    set i i + 1
  ]
end

to setup-globals
  set exits []
  set explosion-areas []
  set casualties 0
  set injured-count 0
  set in-shelter-top-count 0
  set in-shelter-bottom-count 0
  set missile-targets []
  set pending-explosions []
  set impact-areas []
end

to setup-buildings
  let building-width 16
  let building-height 24

  let b1x (min-pxcor + building-width)
  let b1y 10

  let b2x (max-pxcor - building-width)
  let b2y 10

  create-building b1x b1y building-width building-height 1
  create-building b2x b2y building-width building-height 2

  set building1-coordinates (list b1x b1y building-width building-height)
  set building2-coordinates (list b2x b2y building-width building-height)
end

to setup-shelter
  ; Create two shelters, top and bottom
  let shelter-width 14
  let shelter-height 6

  ; Bottom shelter
  let shelter-bottom-x 0
  let shelter-bottom-y (min-pycor + 8)

  ; Top shelter
  let shelter-top-x 0
  let shelter-top-y (max-pycor - 8)

  ; Store coordinates for both shelters
  set shelter-coordinates-bottom (list shelter-bottom-x shelter-bottom-y shelter-width shelter-height)
  set shelter-coordinates-top (list shelter-top-x shelter-top-y shelter-width shelter-height)

  ; Create bottom shelter visuals
  ask patches with [
    pxcor >= shelter-bottom-x - (shelter-width / 2) and
    pxcor <= shelter-bottom-x + (shelter-width / 2) and
    pycor >= shelter-bottom-y - (shelter-height / 2) and
    pycor <= shelter-bottom-y + (shelter-height / 2)
  ] [
    set pcolor gray + 2
  ]

  ; Create top shelter visuals
  ask patches with [
    pxcor >= shelter-top-x - (shelter-width / 2) and
    pxcor <= shelter-top-x + (shelter-width / 2) and
    pycor >= shelter-top-y - (shelter-height / 2) and
    pycor <= shelter-top-y + (shelter-height / 2)
  ] [
    set pcolor gray + 2
  ]

  ; Create shelter labels
  create-floor-labels 1 [
    setxy shelter-bottom-x (shelter-bottom-y + 1)
    set label "SHELTER"
    set color black
  ]

  create-floor-labels 1 [
    setxy shelter-top-x (shelter-top-y + 1)
    set label "SHELTER"
    set color black
  ]
end

to setup-people
  let total-occupants initial-people
  let occupants-per-building floor (total-occupants / 2)

  ; Calculate base number of people per floor (minimum)
  let base-per-floor floor (occupants-per-building / number-of-floors)
  ; Calculate remaining people to distribute randomly
  let remaining-building1 (occupants-per-building - (base-per-floor * number-of-floors))

  ; Building 1 population distribution
  let building1-floor-counts []
  repeat number-of-floors [
    set building1-floor-counts lput base-per-floor building1-floor-counts
  ]

  ; Distribute remaining people randomly across floors for building 1
  repeat remaining-building1 [
    let floor-index random number-of-floors
    let current-count item floor-index building1-floor-counts
    set building1-floor-counts replace-item floor-index building1-floor-counts (current-count + 1)
  ]

  ; Building 2 calculations and similar distribution
  let occupants-building2 (total-occupants - occupants-per-building)
  let base-per-floor2 floor (occupants-building2 / number-of-floors)
  let remaining-building2 (occupants-building2 - (base-per-floor2 * number-of-floors))

  let building2-floor-counts []
  repeat number-of-floors [
    set building2-floor-counts lput base-per-floor2 building2-floor-counts
  ]

  repeat remaining-building2 [
    let floor-index random number-of-floors
    let current-count item floor-index building2-floor-counts
    set building2-floor-counts replace-item floor-index building2-floor-counts (current-count + 1)
  ]

  ; Create people in both buildings using the floor counts
  foreach range number-of-floors [ floor1 ->
    create-people-for-building floor1 building1-floor-counts building1-coordinates 1
    create-people-for-building floor1 building2-floor-counts building2-coordinates 2
  ]

end

to setup-person-properties [floor1 building-num]
  set shape "person"
  set color green
  set size 1
  set health 100
  set floor-number floor1
  set in-building? true
  set in-shelter? false
  set building-number building-num
  set evacuation-speed 0.5 + random-float 0.5
  set reacting? reaction-percentage >= 100 or random 100 < reaction-percentage  ; FIXED VERSION
  set waiting-time 0
  set switch-chance random-float 0.5
  set following-who nobody
  set next-exit nobody
  set chosen-shelter ifelse-value (random 2 = 0) ["top"] ["bottom"]
  set shelter-attempt-count 0
  set waiting-at-shelter? false
  set evacuation-decision "none"
  set evacuation-timing "none"
end

to create-people-for-building [floor1 floor-counts building-coords building-num]
  let people-on-floor item floor1 floor-counts
  let x-pos item 0 building-coords
  let y-pos item 1 building-coords
  let width item 2 building-coords
  let height item 3 building-coords

  create-building-occupants people-on-floor [
    ; Calculate floor position
    let floor-height height / number-of-floors
    let y-base y-pos - (height / 2) + (floor1 * floor-height)

    ; Random position within floor bounds
    setxy (x-pos - (width / 2) + 1 + random-float (width - 2))
          (y-base + 1 + random-float (floor-height - 2))

    setup-person-properties floor1 building-num
  ]
end

to create-building [x y width height building-number1]
  let floor-height height / number-of-floors
  let exits-per-floor number-of-exits

  ; Create floors with different colors
  foreach range number-of-floors [ floor1 ->
    let floor-color item floor1 floor-colors
    let y-base y - (height / 2) + (floor1 * floor-height)

    ; Create floor patches with distinct colors
    ask patches with [
      pxcor >= x - (width / 2) and
      pxcor <= x + (width / 2) and
      pycor >= y-base and
      pycor < y-base + floor-height
    ] [
      set pcolor approximate-rgb (item 0 floor-color)
                                (item 1 floor-color)
                                (item 2 floor-color)
    ]

    ; Create floor label
    create-floor-labels 1 [
      setxy (x - (width / 2) + 2) (y-base + floor-height / 2)
      set label (floor1 + 1)
      set color black
    ]

    ; Create exits for this floor
    let exit-spacing width / (exits-per-floor + 1)
    let exit-num 1

    repeat exits-per-floor [
      let exit-x x - (width / 2) + (exit-spacing * exit-num)
      let exit-y y-base
      set exit-num exit-num + 1

      ; Create door frame
      create-exit-visuals 1 [
        setxy exit-x exit-y
        set color brown - 2
        set shape "square"
        set size 2
        set floor-number floor1
        set building-number building-number1
      ]

      ; Create door
      create-exit-visuals 1 [
        setxy exit-x exit-y
        set color brown
        set shape "square"
        set size 1.8
        set floor-number floor1
        set building-number building-number1
      ]

      ; Store exit coordinates
      set exits lput (list exit-x exit-y floor1 building-number1) exits
    ]
  ]
end

; Distance reporter for building placement
to-report distance-between [x1 y1 x2 y2]
  report sqrt ((x1 - x2) ^ 2 + (y1 - y2) ^ 2)
end

; Distance to exit reporter
to-report distance-to-exit [occupant exit-coords]
  let exit-x item 0 exit-coords
  let exit-y item 1 exit-coords
  report distancexy exit-x exit-y
end

to move-people
  ask building-occupants with [health > 0 and reacting?] [
    ifelse in-building? [
      handle-exit-movement
    ][
      move-towards-shelter    ; This will now be called once they exit
    ]
  ]
end

to record-decisions
  ask building-occupants [
    ; first round
    if evacuation-decision = "none" [
      ifelse reacting? [
        ; First tick reactors are early evacuees
        set evacuation-decision "shelter"

        ifelse first-hit = false [
          set evacuation-timing "early"
          set early-evacuees early-evacuees + 1
        ][
        ; Everyone else starts as stayer
          set evacuation-timing "late"
          set late-evacuees late-evacuees + 1
      ]
    ][
        set evacuation-decision "stay"
        set building-stayers building-stayers + 1
    ]
]
      ; those who changed their mind after the first explosions
      if evacuation-decision = "stay" and reacting?[
        set evacuation-decision "shelter"
        set evacuation-timing "late"
        set building-stayers building-stayers - 1  ; Reduce stayer count
        set late-evacuees late-evacuees + 1
      ]
  ]

end

to handle-exit-movement
  ; Get current floor exits
  let current-floor-exits filter [exit ->
    item 2 exit = floor-number and
    item 3 exit = building-number
  ] exits

  if not empty? current-floor-exits [
    let nearest-exit get-nearest-exit current-floor-exits

    if nearest-exit != nobody [
      let exit-x item 0 nearest-exit
      let exit-y item 1 nearest-exit

      ; Move towards exit
      face patch exit-x exit-y

      ; Check if path is blocked
      let ahead-patch patch-ahead evacuation-speed

      ifelse ahead-patch != nobody and count building-occupants-on ahead-patch > 2 [
        ; Path is blocked - try to move sideways
        ifelse random 2 = 0 [rt 90] [lt 90]
        fd 0.1
      ][
        ; Path is clear - move forward
        fd min (list evacuation-speed 0.3)
      ]

      ; Handle exit transition when close enough
      if distancexy exit-x exit-y < 1 [
        handle-exit-transition
      ]
    ]
  ]
end

to handle-exit-transition
  ifelse floor-number > 0 [
    ; Move to lower floor
    set floor-number floor-number - 1
    move-to-lower-floor
    set next-exit nobody  ; Reset exit choice for new floor
  ][
    ; Exit building when on ground floor
    set in-building? false
    ; Place person just outside the building
    let exit-x xcor
    let building-width 16  ; from setup-buildings
    if building-number = 1 [
      setxy (exit-x + 2) ycor  ; Move right if building 1
    ]
    if building-number = 2 [
      setxy (exit-x - 2) ycor  ; Move left if building 2
    ]
    set next-exit nobody
  ]
end

; First, get only downward exits for current floor
to-report get-downward-exits [floor-exits]
  let downward-exits []

  foreach floor-exits [ this-exit ->
    let exit-floor item 2 this-exit
    let exit-building item 3 this-exit
    let this-x item 0 this-exit   ; Get x coordinate of current exit

    ifelse exit-floor = 0 [
      ; If it's a ground floor exit, it leads outside - add it
      set downward-exits lput this-exit downward-exits
    ][
      ; For other floors, check if there's an exit below
      let connected-exit nobody
      foreach exits [ lower-exit ->
        let lower-x item 0 lower-exit
        let lower-y item 1 lower-exit
        let lower-floor item 2 lower-exit
        let lower-building item 3 lower-exit

        ; If exit is directly below (same x coordinate, one floor down)
        if lower-building = exit-building and
           lower-floor = (exit-floor - 1) and
           abs (lower-x - this-x) < 1 [  ; Compare with this-x
          set connected-exit lower-exit
        ]
      ]

      ; If this exit connects to a lower floor, add it
      if connected-exit != nobody [
        set downward-exits lput this-exit downward-exits
      ]
    ]
  ]

  report downward-exits
end

; Then find nearest among downward exits
to-report get-nearest-exit [floor-exits]
  ; First get only downward exits
  let downward-exits get-downward-exits floor-exits

  ; If no downward exits found, report nobody
  if empty? downward-exits [
    report nobody
  ]

  ; Find nearest among downward exits
  let closest-exit nobody
  let min-distance 999999

  foreach downward-exits [ this-exit ->
    let dist distance-to-exit self this-exit
    if dist < min-distance [
      set min-distance dist
      set closest-exit this-exit
    ]
  ]

  report closest-exit
end

to move-towards-shelter
  if not in-shelter? [
    ; Only move if not already waiting at a shelter
    if not waiting-at-shelter? [
      let target-shelter ifelse-value (chosen-shelter = "top")
        [shelter-coordinates-top]
        [shelter-coordinates-bottom]

      let shelter-x item 0 target-shelter
      let shelter-y item 1 target-shelter
      let target-patch patch shelter-x shelter-y

      if target-patch != nobody [
        ; If close to shelter entrance
        ifelse distance target-patch < 3 [
          let is-full? shelter-at-capacity? chosen-shelter

          ifelse not is-full? [
            ; Enter shelter - move to random shelter patch
            let shelter-patches patches with [
              pcolor = gray + 2 and
              (ifelse-value ([chosen-shelter] of myself = "top")
                [pycor > 0]
                [pycor < 0])
            ]

            if any? shelter-patches [
              move-to one-of shelter-patches
              set in-shelter? true
              set color green
            ]
          ][
            ; Shelter is full - make decision once
            ; Move slightly away from shelter entrance
            rt 90 + random 180
            fd 2
            set waiting-at-shelter? true
            set shelter-attempt-count shelter-attempt-count + 1

            ; Only try other shelter if haven't tried both yet
            if shelter-attempt-count = 1 [
              ; 50% chance to try other shelter
              if random 100 < 50 [
                set chosen-shelter ifelse-value (chosen-shelter = "top") ["bottom"] ["top"]
                set color blue  ; Visual indicator of shelter switch
                set waiting-at-shelter? false  ; Allow movement to new shelter
              ]
            ]
          ]
        ][
          ; Still approaching shelter
          face target-patch
          fd min (list evacuation-speed 0.3)
        ]
      ]
    ]
  ]
end

to move-to-lower-floor
  let current-building building-number

  ; Find exits on the new floor
  let new-floor-exits filter [exit ->
    item 2 exit = floor-number and
    item 3 exit = current-building
  ] exits

  if not empty? new-floor-exits [
    ; Choose random exit on new floor and move near it
    let target-exit one-of new-floor-exits
    setxy (item 0 target-exit + random-float 2 - 1)
          (item 1 target-exit + random-float 2)
  ]
end

to move-missiles
  ask missiles [
    ; Move toward impact point
    face impact-point
    fd speed

    ; Check for impact
    if distance impact-point < 1 [
      ; Add explosion info to pending list instead of creating directly
      set pending-explosions lput (list xcor ycor explosion-radius) pending-explosions
      die
    ]
  ]

  ; Handle any pending explosions from observer context
  handle-pending-explosions
end

to handle-pending-explosions
  foreach pending-explosions [ explosion-info ->
    let x-pos item 0 explosion-info
    let y-pos item 1 explosion-info
    let exp-radius item 2 explosion-info

    ; Create the explosion effect
    create-explosions 1 [
      setxy x-pos y-pos
      set shape "circle"
      set color 27  ; Light orange
      set size 2    ; Reduced from 3 to 2
      set duration 10
      set radius exp-radius
      set explosion-areas lput (list xcor ycor radius) explosion-areas
    ]

    ; Create permanent impact visualization
    let impact-center patch x-pos y-pos
    let max-distance exp-radius

    ask impact-center [
      ask patches in-radius max-distance [
        let distance-from-center distance impact-center
        let impact-intensity (1 - (distance-from-center / max-distance))

        if impact-intensity > 0.15 [  ; Increased threshold slightly
          let red-component 200 + (55 * (1 - impact-intensity))
          let other-components 100 * (1 - impact-intensity)
          set pcolor approximate-rgb red-component other-components other-components

          if impact-intensity > 0.8 [
            set pcolor approximate-rgb (red-component * 0.7) (other-components * 0.7) (other-components * 0.7)
          ]
        ]
      ]
    ]

    set impact-areas lput (list x-pos y-pos exp-radius) impact-areas
  ]
  set pending-explosions []
end

to create-explosion-at [x y radius1]  ; Observer procedure
  create-explosions 1 [
    setxy x y
    set shape "circle"
    set color orange
    set size 3
    set duration 10
    set radius radius1

    ; Add to tracking list
    set explosion-areas lput (list xcor ycor radius) explosion-areas
  ]
end

to go
  if simulation-complete [stop]

  ; Check for first missile hit and update reaction percentage
  if not first-hit and any? explosions [
    set first-hit true
    adjust-reaction-percentage
  ]

  record-decisions
  move-people
  handle-missiles
  move-missiles
  handle-explosions
  update-casualties

  if missiles-launched >= number-of-missiles and
     not any? missiles and
     not any? explosions [
    set simulation-complete true
  ]

  tick
end

to handle-missiles
  if missile-strategy = "immediate" [
    launch-immediate-missiles
  ]
  if missile-strategy = "delayed" [
    launch-delayed-missiles
  ]
end

to launch-immediate-missiles
  if ticks = 10 and missiles-launched < number-of-missiles [
    repeat number-of-missiles [
      create-missile
      set missiles-launched missiles-launched + 1
    ]
  ]
end

to launch-delayed-missiles
  if ticks mod missile-delay = 0 and ticks >= 10 [
    if missiles-launched < number-of-missiles [
      create-missile
      set missiles-launched missiles-launched + 1
    ]
  ]
end

to create-missile
  create-missiles 1 [
    set shape "arrow"
    set color red
    set size 2

    ; Create a target point based on strategy
    let target-patch nobody
    ifelse missile-strategy = "immediate"
    [ ; Target buildings directly
      set target-patch one-of patches with [
        abs pxcor < 35 and
        abs pycor < 15
      ]
    ]
    [ ; For delayed strategy, alternate between buildings and yard
      ifelse count missiles mod 2 = 0
      [ ; Target buildings
        set target-patch one-of patches with [
          abs pxcor < 35 and
          abs pycor < 15
        ]
      ]
      [ ; Target yard areas
        set target-patch one-of patches with [
          abs pxcor < 45 and  ; Wider area for yard
          abs pycor < 20      ; Larger area for yard
        ]
      ]
    ]

    set impact-point target-patch
    setxy max-pxcor random-ycor  ; Start from right edge
    set explosion-radius 6    ; Reduced from 10 to 6
    set speed 0.5
  ]
end

to set-building-target  ; For missiles breed
  let target-patch one-of patches with [
    (abs pxcor < 35) and (abs pycor < 15)
  ]
  set impact-point target-patch
end

to set-yard-target  ; For missiles breed
  let target-patch one-of patches with [
    (abs pxcor < 25) and (abs pycor < 15)
  ]
  set impact-point target-patch
end

to handle-explosions
  ask explosions [
    set duration duration - 1
    if duration <= 0 [ die ]

    let current-radius radius

    ask building-occupants with [health > 0] in-radius current-radius [
      let damage 0

      ifelse in-shelter? [
        ; Reduced damage in shelters
        ifelse chosen-shelter = "top" [
          ; Top shelter might have slightly different protection based on location
          set damage 4
        ] [
          ; Bottom shelter
          set damage 5
        ]
      ] [
        if in-building? [ set damage 40 ]
        if (not in-building?) and (not in-shelter?) [ set damage 80 ]
      ]

      let distance-factor 1 - (distance myself / current-radius)
      set damage damage * distance-factor

      set health health - damage
      update-person-color
    ]

    set size size + 0.1
    set color 27 + (1.5 * (10 - duration))
  ]
end

; Add to setup procedure - reset patches to background color
to-report get-floor-color [floor-num]
  report item floor-num floor-colors
end

to reset-patch-colors
  ask patches [
    ; Reset to appropriate floor color if in building
    let floor-num floor ((pycor - min-pycor) / 6)  ; Adjust based on your floor height
    if floor-num >= 0 and floor-num < 4 [
      set pcolor white
    ]
  ]
end

to update-person-color
  if health <= 0 [ set color black ]
  if health > 0 and health < 50 [ set color red ]
  if health >= 50 and health < 80 [ set color yellow ]
  if health >= 80 [ set color green ]
end

to update-casualties
  set casualties count building-occupants with [health <= 0]
  set injured-count count building-occupants with [health > 0 and health < 80]
end

to adjust-reaction-percentage
  ; Increase reaction percentage to 95% after first hit
  let new-percentage 95

  ask building-occupants with [not reacting? and health > 0] [
    if random 100 < new-percentage [
      set reacting? true
    ]
  ]
end

to-report death-count
  report casualties
end

to-report injured-people
  report injured-count
end

to-report missiles-remaining
  report number-of-missiles - missiles-launched
end

to-report shelter-status
  report (word
    "Top Shelter: " shelter-top-occupants "/" shelter-capacity
    " | Bottom Shelter: " shelter-bottom-occupants "/" shelter-capacity)
end

to-report shelter-at-capacity? [shelter-type]
  let shelter-patches patches with [
    pcolor = gray + 2 and
    (ifelse-value (shelter-type = "top")
      [pycor > 0]
      [pycor < 0])
  ]

  let current-count count building-occupants with [
    in-shelter? and
    chosen-shelter = shelter-type and
    member? patch-here shelter-patches
  ]

  ; Update global counters
  ifelse shelter-type = "top"
    [set shelter-top-occupants current-count]
    [set shelter-bottom-occupants current-count]

  report current-count >= shelter-capacity
end

; Reporters for plotting
to-report early-evacuee-casualties
  report count building-occupants with [
    evacuation-timing = "early" and
    health <= 0
  ]
end

to-report early-evacuee-injured
  report count building-occupants with [
    evacuation-timing = "early" and
    health > 0 and health < 80
  ]
end

to-report early-evacuee-intact
  report count building-occupants with [
    evacuation-timing = "early" and
    health >= 80
  ]
end

to-report late-evacuee-casualties
  report count building-occupants with [
    evacuation-timing = "late" and
    health <= 0
  ]
end

to-report late-evacuee-injured
  report count building-occupants with [
    evacuation-timing = "late" and
    health > 0 and health < 80
  ]
end

to-report late-evacuee-intact
  report count building-occupants with [
    evacuation-timing = "late" and
    health >= 80
  ]
end

to-report stayer-casualties
  report count building-occupants with [
    evacuation-decision = "stay" and
    health <= 0
  ]
end

to-report stayer-injured
  report count building-occupants with [
    evacuation-decision = "stay" and
    health > 0 and health < 80
  ]
end

to-report stayer-intact
  report count building-occupants with [
    evacuation-decision = "stay" and
    health >= 80
  ]
end

; Calculate percentages for plotting
to-report early-evacuee-casualty-percent
  ifelse early-evacuees > 0 [
    report (early-evacuee-casualties / early-evacuees) * 100
  ][
    report 0
  ]
end

to-report early-evacuee-injured-percent
  ifelse early-evacuees > 0 [
    report (early-evacuee-injured / early-evacuees) * 100
  ][
    report 0
  ]
end

to-report late-evacuee-casualty-percent
  ifelse late-evacuees > 0 [
    report (late-evacuee-casualties / late-evacuees) * 100
  ][
    report 0
  ]
end

to-report late-evacuee-injured-percent
  ifelse late-evacuees > 0 [
    report (late-evacuee-injured / late-evacuees) * 100
  ][
    report 0
  ]
end

to-report stayer-casualty-percent
  ifelse building-stayers > 0 [
    report (stayer-casualties / building-stayers) * 100
  ][
    report 0
  ]
end

to-report stayer-injured-percent
  ifelse building-stayers > 0 [
    report (stayer-injured / building-stayers) * 100
  ][
    report 0
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
210
10
623
324
-1
-1
5.0
1
10
1
1
1
0
0
0
1
-40
40
-30
30
0
0
1
ticks
30.0

BUTTON
15
20
78
53
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
85
20
148
53
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
9
72
181
105
initial-people
initial-people
100
500
500.0
50
1
NIL
HORIZONTAL

SLIDER
8
196
180
229
number-of-exits
number-of-exits
1
5
2.0
1
1
NIL
HORIZONTAL

SLIDER
7
368
179
401
missile-delay
missile-delay
10
100
50.0
10
1
NIL
HORIZONTAL

SLIDER
8
282
180
315
number-of-missiles
number-of-missiles
1
10
10.0
1
1
NIL
HORIZONTAL

SLIDER
8
230
180
263
reaction-percentage
reaction-percentage
0
100
19.0
1
1
NIL
HORIZONTAL

CHOOSER
7
320
171
365
missile-strategy
missile-strategy
"immediate" "delayed"
0

MONITOR
625
10
683
55
Dead
death-count
0
1
11

MONITOR
625
55
683
100
Injured
injured-people
17
1
11

MONITOR
695
100
755
145
In Shelter
count building-occupants with [in-shelter?]
0
1
11

MONITOR
695
10
755
55
At Home
count building-occupants with [in-building?]
17
1
11

MONITOR
695
54
755
99
In Transit
count building-occupants with [in-shelter? = false and in-building? = false]
0
1
11

SLIDER
8
160
181
193
number-of-floors
number-of-floors
2
10
5.0
1
1
NIL
HORIZONTAL

SLIDER
9
108
182
141
shelter-capacity
shelter-capacity
50
500
50.0
50
1
NIL
HORIZONTAL

MONITOR
310
307
534
340
Shelters Status
shelter-status
17
1
8

PLOT
627
158
827
308
Early Evacuation
Time
Percent
0.0
10.0
0.0
100.0
true
true
"" ""
PENS
"Dead" 1.0 0 -2674135 true "" "plot early-evacuee-casualty-percent"
"Injured" 1.0 0 -1184463 true "" "plot early-evacuee-injured-percent"
"Intact" 1.0 0 -13840069 true "" "plot 100 - early-evacuee-casualty-percent - early-evacuee-injured-percent"

PLOT
624
308
824
458
Late Evacuation
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Dead" 1.0 0 -2674135 true "" "plot late-evacuee-casualty-percent "
"Injured" 1.0 0 -1184463 true "" "plot late-evacuee-injured-percent"
"Intact" 1.0 0 -13840069 true "" "plot 100 - late-evacuee-casualty-percent - late-evacuee-injured-percent"

PLOT
828
158
1028
308
Building Stayers
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"Dead" 1.0 0 -2674135 true "" "plot stayer-casualty-percent"
"Injured" 1.0 0 -1184463 true "" "plot stayer-injured-percent"
"Intact" 1.0 0 -13840069 true "" "plot 100 - stayer-casualty-percent - stayer-injured-percent"

MONITOR
786
9
844
55
Stayers
building-stayers
17
1
11

MONITOR
787
55
864
101
Early Leavers
early-evacuees
17
1
11

MONITOR
788
100
863
146
Late Leavers
late-evacuees
17
1
11

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.4.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
